@@ -1,7 +1,7 @@
 // qmidictl.cpp
 //
 /****************************************************************************
-   Copyright (C) 2010-2024, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2010-2025, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -170,13 +170,7 @@ int main ( int argc, char *argv[] )
 	app.setDesktopFileName(
 		QString("org.rncbc.%1").arg(PROJECT_NAME));
 #endif
-	QString sVersion(PROJECT_VERSION);
-	sVersion += '\n';
-	sVersion += QString("Qt: %1").arg(qVersion());
-#if defined(QT_STATIC)
-	sVersion += "-static";
-#endif
-	QApplication::setApplicationVersion(sVersion);
+	QApplication::setApplicationVersion(PROJECT_VERSION);
 #endif
 
 	qmidictlOptions opt;
