@@ -1,7 +1,7 @@
 // qmidictlMidiControl.cpp
 //
 /****************************************************************************
-   Copyright (C) 2010, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2010-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -24,13 +24,17 @@
 #include <QStringList>
 #include <QSettings>
 
+// Translatable macro contextualizer.
+#undef  _TR
+#define _TR(x) QT_TR_NOOP(x)
+
 
 //----------------------------------------------------------------------
 // qmidictlMidiControl -- MIDI control map (singleton).
 //
 
 // Kind of singleton reference.
-qmidictlMidiControl* qmidictlMidiControl::g_pMidiControl = NULL;
+qmidictlMidiControl* qmidictlMidiControl::g_pMidiControl = nullptr;
 
 // Constructor.
 qmidictlMidiControl::qmidictlMidiControl (void)
@@ -47,7 +51,7 @@ qmidictlMidiControl::qmidictlMidiControl (void)
 qmidictlMidiControl::~qmidictlMidiControl (void)
 {
 	// Pseudo-singleton reference shut-down.
-	g_pMidiControl = NULL;
+	g_pMidiControl = nullptr;
 }
 
 
@@ -69,13 +73,13 @@ void qmidictlMidiControl::clear (void)
 	mapCommand(REC,        MMC, 0, int(REC));
 	mapCommand(FFWD,       MMC, 0, int(FFWD));
 
-	mapCommand(JOG_WHEEL,  MMC, 0, int(JOG_WHEEL));
-
 	mapCommand(TRACK_SOLO, MMC, 0, int(TRACK_SOLO));
 	mapCommand(TRACK_MUTE, MMC, 0, int(TRACK_MUTE));
 	mapCommand(TRACK_REC,  MMC, 0, int(TRACK_REC));
 
-	mapCommand(TRACK_VOL, CONTROLLER, 15, TrackParam | 0x40);
+	mapCommand(TRACK_VOL,  CONTROLLER, 15, TrackParam | 0x40);
+
+	mapCommand(JOG_WHEEL,  MMC, 0, int(JOG_WHEEL));
 }
 
 
@@ -225,6 +229,23 @@ void qmidictlMidiControl::load ( QSettings& settings )
 
 
 // Textual helpers.
+unsigned short qmidictlMidiControl::keyFromText ( const QString& sText )
+{
+	if (sText == "TrackParam" || sText == "*" || sText.isEmpty())
+		return TrackParam;
+	else
+		return sText.toUShort();
+}
+
+QString qmidictlMidiControl::textFromKey ( unsigned short iKey )
+{
+	if (iKey & TrackParam)
+		return "*"; // "TrackParam";
+	else
+		return QString::number(iKey);
+}
+
+
 qmidictlMidiControl::ControlType qmidictlMidiControl::typeFromText (
 	const QString& sText )
 {
@@ -291,23 +312,6 @@ QString qmidictlMidiControl::textFromType (
 }
 
 
-unsigned short qmidictlMidiControl::keyFromText ( const QString& sText )
-{
-	if (sText == "TrackParam" || sText == "*" || sText.isEmpty())
-		return TrackParam;
-	else
-		return sText.toUShort();
-}
-
-QString qmidictlMidiControl::textFromKey ( unsigned short iKey )
-{
-	if (iKey & TrackParam)
-		return "*"; // "TrackParam";
-	else
-		return QString::number(iKey);
-}
-
-
 qmidictlMidiControl::Command qmidictlMidiControl::commandFromText (
 	const QString& sText )
 {
@@ -329,9 +333,6 @@ qmidictlMidiControl::Command qmidictlMidiControl::commandFromText (
 	if (sText == "FFWD")
 		return FFWD;
 	else
-	if (sText == "JOG_WHEEL")
-		return JOG_WHEEL;
-	else
 	if (sText == "TRACK_SOLO")
 		return TRACK_SOLO;
 	else
@@ -343,6 +344,9 @@ qmidictlMidiControl::Command qmidictlMidiControl::commandFromText (
 	else
 	if (sText == "TRACK_VOL")
 		return TRACK_VOL;
+	else
+	if (sText == "JOG_WHEEL")
+		return JOG_WHEEL;
 	else
 		return Command(0);
 }
@@ -371,9 +375,6 @@ QString qmidictlMidiControl::textFromCommand (
 	case FFWD:
 		sText = "FFWD";
 		break;
-	case JOG_WHEEL:
-		sText = "JOG_WHEEL";
-		break;
 	case TRACK_SOLO:
 		sText = "TRACK_SOLO";
 		break;
@@ -386,6 +387,9 @@ QString qmidictlMidiControl::textFromCommand (
 	case TRACK_VOL:
 		sText = "TRACK_VOL";
 		break;
+	case JOG_WHEEL:
+		sText = "JOG_WHEEL";
+		break;
 	}
 
 	return sText;
@@ -403,69 +407,69 @@ static struct
 } g_aNoteNames[] = {
 
 	// Diatonic note map...
-	{  0, QT_TR_NOOP("C")  },
-	{  1, QT_TR_NOOP("C#") },
-	{  2, QT_TR_NOOP("D")  },
-	{  3, QT_TR_NOOP("D#") },
-	{  4, QT_TR_NOOP("E")  },
-	{  5, QT_TR_NOOP("F")  },
-	{  6, QT_TR_NOOP("F#") },
-	{  7, QT_TR_NOOP("G")  },
-	{  8, QT_TR_NOOP("G#") },
-	{  9, QT_TR_NOOP("A")  },
-	{ 10, QT_TR_NOOP("A#") },
-	{ 11, QT_TR_NOOP("B")  },
+	{  0, _TR("C")     },
+	{  1, _TR("C#/Db") },
+	{  2, _TR("D")     },
+	{  3, _TR("D#/Eb") },
+	{  4, _TR("E")     },
+	{  5, _TR("F")     },
+	{  6, _TR("F#/Gb") },
+	{  7, _TR("G")     },
+	{  8, _TR("G#/Ab") },
+	{  9, _TR("A")     },
+	{ 10, _TR("A#/Bb") },
+	{ 11, _TR("B")     },
 
 	// GM Drum note map...
-	{ 35, QT_TR_NOOP("Acoustic Bass Drum") },
-	{ 36, QT_TR_NOOP("Bass Drum 1") },
-	{ 37, QT_TR_NOOP("Side Stick") },
-	{ 38, QT_TR_NOOP("Acoustic Snare") },
-	{ 39, QT_TR_NOOP("Hand Clap") },
-	{ 40, QT_TR_NOOP("Electric Snare") },
-	{ 41, QT_TR_NOOP("Low Floor Tom") },
-	{ 42, QT_TR_NOOP("Closed Hi-Hat") },
-	{ 43, QT_TR_NOOP("High Floor Tom") },
-	{ 44, QT_TR_NOOP("Pedal Hi-Hat") },
-	{ 45, QT_TR_NOOP("Low Tom") },
-	{ 46, QT_TR_NOOP("Open Hi-Hat") },
-	{ 47, QT_TR_NOOP("Low-Mid Tom") },
-	{ 48, QT_TR_NOOP("Hi-Mid Tom") },
-	{ 49, QT_TR_NOOP("Crash Cymbal 1") },
-	{ 50, QT_TR_NOOP("High Tom") },
-	{ 51, QT_TR_NOOP("Ride Cymbal 1") },
-	{ 52, QT_TR_NOOP("Chinese Cymbal") },
-	{ 53, QT_TR_NOOP("Ride Bell") },
-	{ 54, QT_TR_NOOP("Tambourine") },
-	{ 55, QT_TR_NOOP("Splash Cymbal") },
-	{ 56, QT_TR_NOOP("Cowbell") },
-	{ 57, QT_TR_NOOP("Crash Cymbal 2") },
-	{ 58, QT_TR_NOOP("Vibraslap") },
-	{ 59, QT_TR_NOOP("Ride Cymbal 2") },
-	{ 60, QT_TR_NOOP("Hi Bongo") },
-	{ 61, QT_TR_NOOP("Low Bongo") },
-	{ 62, QT_TR_NOOP("Mute Hi Conga") },
-	{ 63, QT_TR_NOOP("Open Hi Conga") },
-	{ 64, QT_TR_NOOP("Low Conga") },
-	{ 65, QT_TR_NOOP("High Timbale") },
-	{ 66, QT_TR_NOOP("Low Timbale") },
-	{ 67, QT_TR_NOOP("High Agogo") },
-	{ 68, QT_TR_NOOP("Low Agogo") },
-	{ 69, QT_TR_NOOP("Cabasa") },
-	{ 70, QT_TR_NOOP("Maracas") },
-	{ 71, QT_TR_NOOP("Short Whistle") },
-	{ 72, QT_TR_NOOP("Long Whistle") },
-	{ 73, QT_TR_NOOP("Short Guiro") },
-	{ 74, QT_TR_NOOP("Long Guiro") },
-	{ 75, QT_TR_NOOP("Claves") },
-	{ 76, QT_TR_NOOP("Hi Wood Block") },
-	{ 77, QT_TR_NOOP("Low Wood Block") },
-	{ 78, QT_TR_NOOP("Mute Cuica") },
-	{ 79, QT_TR_NOOP("Open Cuica") },
-	{ 80, QT_TR_NOOP("Mute Triangle") },
-	{ 81, QT_TR_NOOP("Open Triangle") },
+	{ 35, _TR("Acoustic Bass Drum") },
+	{ 36, _TR("Bass Drum 1") },
+	{ 37, _TR("Side Stick") },
+	{ 38, _TR("Acoustic Snare") },
+	{ 39, _TR("Hand Clap") },
+	{ 40, _TR("Electric Snare") },
+	{ 41, _TR("Low Floor Tom") },
+	{ 42, _TR("Closed Hi-Hat") },
+	{ 43, _TR("High Floor Tom") },
+	{ 44, _TR("Pedal Hi-Hat") },
+	{ 45, _TR("Low Tom") },
+	{ 46, _TR("Open Hi-Hat") },
+	{ 47, _TR("Low-Mid Tom") },
+	{ 48, _TR("Hi-Mid Tom") },
+	{ 49, _TR("Crash Cymbal 1") },
+	{ 50, _TR("High Tom") },
+	{ 51, _TR("Ride Cymbal 1") },
+	{ 52, _TR("Chinese Cymbal") },
+	{ 53, _TR("Ride Bell") },
+	{ 54, _TR("Tambourine") },
+	{ 55, _TR("Splash Cymbal") },
+	{ 56, _TR("Cowbell") },
+	{ 57, _TR("Crash Cymbal 2") },
+	{ 58, _TR("Vibraslap") },
+	{ 59, _TR("Ride Cymbal 2") },
+	{ 60, _TR("Hi Bongo") },
+	{ 61, _TR("Low Bongo") },
+	{ 62, _TR("Mute Hi Conga") },
+	{ 63, _TR("Open Hi Conga") },
+	{ 64, _TR("Low Conga") },
+	{ 65, _TR("High Timbale") },
+	{ 66, _TR("Low Timbale") },
+	{ 67, _TR("High Agogo") },
+	{ 68, _TR("Low Agogo") },
+	{ 69, _TR("Cabasa") },
+	{ 70, _TR("Maracas") },
+	{ 71, _TR("Short Whistle") },
+	{ 72, _TR("Long Whistle") },
+	{ 73, _TR("Short Guiro") },
+	{ 74, _TR("Long Guiro") },
+	{ 75, _TR("Claves") },
+	{ 76, _TR("Hi Wood Block") },
+	{ 77, _TR("Low Wood Block") },
+	{ 78, _TR("Mute Cuica") },
+	{ 79, _TR("Open Cuica") },
+	{ 80, _TR("Mute Triangle") },
+	{ 81, _TR("Open Triangle") },
 
-	{  0, NULL }
+	{  0, nullptr }
 };
 
 static QHash<unsigned short, QString> g_noteNames;
@@ -478,9 +482,8 @@ const QString qmidictlMidiControl::noteName (
 		// Pre-load drum-names hash table...
 		if (g_noteNames.isEmpty()) {
 			for (int i = 12; g_aNoteNames[i].name; ++i) {
-				g_noteNames.insert(
-					g_aNoteNames[i].note,
-					QObject::tr(g_aNoteNames[i].name));
+				g_noteNames.insert(g_aNoteNames[i].note,
+					QObject::tr(g_aNoteNames[i].name, "noteName"));
 			}
 		}
 		// Check whether the drum note exists...
@@ -490,7 +493,7 @@ const QString qmidictlMidiControl::noteName (
 			return iter.value();
 	}
 
-	return QObject::tr(g_aNoteNames[iParam % 12].name)
+	return QObject::tr(g_aNoteNames[iParam % 12].name, "noteName")
 		+ QString::number((iParam / 12) - 2);
 }
 
@@ -505,75 +508,75 @@ static struct
 
 } g_aControllerNames[] = {
 
-	{   0, QT_TR_NOOP("Bank Select (coarse)") },
-	{   1, QT_TR_NOOP("Modulation Wheel (coarse)") },
-	{   2, QT_TR_NOOP("Breath Controller (coarse)") },
-	{   4, QT_TR_NOOP("Foot Pedal (coarse)") },
-	{   5, QT_TR_NOOP("Portamento Time (coarse)") },
-	{   6, QT_TR_NOOP("Data Entry (coarse)") },
-	{   7, QT_TR_NOOP("Volume (coarse)") },
-	{   8, QT_TR_NOOP("Balance (coarse)") },
-	{  10, QT_TR_NOOP("Pan Position (coarse)") },
-	{  11, QT_TR_NOOP("Expression (coarse)") },
-	{  12, QT_TR_NOOP("Effect Control 1 (coarse)") },
-	{  13, QT_TR_NOOP("Effect Control 2 (coarse)") },
-	{  16, QT_TR_NOOP("General Purpose Slider 1") },
-	{  17, QT_TR_NOOP("General Purpose Slider 2") },
-	{  18, QT_TR_NOOP("General Purpose Slider 3") },
-	{  19, QT_TR_NOOP("General Purpose Slider 4") },
-	{  32, QT_TR_NOOP("Bank Select (fine)") },
-	{  33, QT_TR_NOOP("Modulation Wheel (fine)") },
-	{  34, QT_TR_NOOP("Breath Controller (fine)") },
-	{  36, QT_TR_NOOP("Foot Pedal (fine)") },
-	{  37, QT_TR_NOOP("Portamento Time (fine)") },
-	{  38, QT_TR_NOOP("Data Entry (fine)") },
-	{  39, QT_TR_NOOP("Volume (fine)") },
-	{  40, QT_TR_NOOP("Balance (fine)") },
-	{  42, QT_TR_NOOP("Pan Position (fine)") },
-	{  43, QT_TR_NOOP("Expression (fine)") },
-	{  44, QT_TR_NOOP("Effect Control 1 (fine)") },
-	{  45, QT_TR_NOOP("Effect Control 2 (fine)") },
-	{  64, QT_TR_NOOP("Hold Pedal (on/off)") },
-	{  65, QT_TR_NOOP("Portamento (on/off)") },
-	{  66, QT_TR_NOOP("Sustenuto Pedal (on/off)") },
-	{  67, QT_TR_NOOP("Soft Pedal (on/off)") },
-	{  68, QT_TR_NOOP("Legato Pedal (on/off)") },
-	{  69, QT_TR_NOOP("Hold 2 Pedal (on/off)") },
-	{  70, QT_TR_NOOP("Sound Variation") },
-	{  71, QT_TR_NOOP("Sound Timbre") },
-	{  72, QT_TR_NOOP("Sound Release Time") },
-	{  73, QT_TR_NOOP("Sound Attack Time") },
-	{  74, QT_TR_NOOP("Sound Brightness") },
-	{  75, QT_TR_NOOP("Sound Control 6") },
-	{  76, QT_TR_NOOP("Sound Control 7") },
-	{  77, QT_TR_NOOP("Sound Control 8") },
-	{  78, QT_TR_NOOP("Sound Control 9") },
-	{  79, QT_TR_NOOP("Sound Control 10") },
-	{  80, QT_TR_NOOP("General Purpose Button 1 (on/off)") },
-	{  81, QT_TR_NOOP("General Purpose Button 2 (on/off)") },
-	{  82, QT_TR_NOOP("General Purpose Button 3 (on/off)") },
-	{  83, QT_TR_NOOP("General Purpose Button 4 (on/off)") },
-	{  91, QT_TR_NOOP("Effects Level") },
-	{  92, QT_TR_NOOP("Tremulo Level") },
-	{  93, QT_TR_NOOP("Chorus Level") },
-	{  94, QT_TR_NOOP("Celeste Level") },
-	{  95, QT_TR_NOOP("Phaser Level") },
-	{  96, QT_TR_NOOP("Data Button Increment") },
-	{  97, QT_TR_NOOP("Data Button Decrement") },
-	{  98, QT_TR_NOOP("Non-Registered Parameter (fine)") },
-	{  99, QT_TR_NOOP("Non-Registered Parameter (coarse)") },
-	{ 100, QT_TR_NOOP("Registered Parameter (fine)") },
-	{ 101, QT_TR_NOOP("Registered Parameter (coarse)") },
-	{ 120, QT_TR_NOOP("All Sound Off") },
-	{ 121, QT_TR_NOOP("All Controllers Off") },
-	{ 122, QT_TR_NOOP("Local Keyboard (on/off)") },
-	{ 123, QT_TR_NOOP("All Notes Off") },
-	{ 124, QT_TR_NOOP("Omni Mode Off") },
-	{ 125, QT_TR_NOOP("Omni Mode On") },
-	{ 126, QT_TR_NOOP("Mono Operation") },
-	{ 127, QT_TR_NOOP("Poly Operation") },
+	{   0, _TR("Bank Select (coarse)") },
+	{   1, _TR("Modulation Wheel (coarse)") },
+	{   2, _TR("Breath Controller (coarse)") },
+	{   4, _TR("Foot Pedal (coarse)") },
+	{   5, _TR("Portamento Time (coarse)") },
+	{   6, _TR("Data Entry (coarse)") },
+	{   7, _TR("Volume (coarse)") },
+	{   8, _TR("Balance (coarse)") },
+	{  10, _TR("Pan Position (coarse)") },
+	{  11, _TR("Expression (coarse)") },
+	{  12, _TR("Effect Control 1 (coarse)") },
+	{  13, _TR("Effect Control 2 (coarse)") },
+	{  16, _TR("General Purpose Slider 1") },
+	{  17, _TR("General Purpose Slider 2") },
+	{  18, _TR("General Purpose Slider 3") },
+	{  19, _TR("General Purpose Slider 4") },
+	{  32, _TR("Bank Select (fine)") },
+	{  33, _TR("Modulation Wheel (fine)") },
+	{  34, _TR("Breath Controller (fine)") },
+	{  36, _TR("Foot Pedal (fine)") },
+	{  37, _TR("Portamento Time (fine)") },
+	{  38, _TR("Data Entry (fine)") },
+	{  39, _TR("Volume (fine)") },
+	{  40, _TR("Balance (fine)") },
+	{  42, _TR("Pan Position (fine)") },
+	{  43, _TR("Expression (fine)") },
+	{  44, _TR("Effect Control 1 (fine)") },
+	{  45, _TR("Effect Control 2 (fine)") },
+	{  64, _TR("Hold Pedal (on/off)") },
+	{  65, _TR("Portamento (on/off)") },
+	{  66, _TR("Sustenuto Pedal (on/off)") },
+	{  67, _TR("Soft Pedal (on/off)") },
+	{  68, _TR("Legato Pedal (on/off)") },
+	{  69, _TR("Hold 2 Pedal (on/off)") },
+	{  70, _TR("Sound Variation") },
+	{  71, _TR("Sound Timbre") },
+	{  72, _TR("Sound Release Time") },
+	{  73, _TR("Sound Attack Time") },
+	{  74, _TR("Sound Brightness") },
+	{  75, _TR("Sound Control 6") },
+	{  76, _TR("Sound Control 7") },
+	{  77, _TR("Sound Control 8") },
+	{  78, _TR("Sound Control 9") },
+	{  79, _TR("Sound Control 10") },
+	{  80, _TR("General Purpose Button 1 (on/off)") },
+	{  81, _TR("General Purpose Button 2 (on/off)") },
+	{  82, _TR("General Purpose Button 3 (on/off)") },
+	{  83, _TR("General Purpose Button 4 (on/off)") },
+	{  91, _TR("Effects Level") },
+	{  92, _TR("Tremulo Level") },
+	{  93, _TR("Chorus Level") },
+	{  94, _TR("Celeste Level") },
+	{  95, _TR("Phaser Level") },
+	{  96, _TR("Data Button Increment") },
+	{  97, _TR("Data Button Decrement") },
+	{  98, _TR("Non-Registered Parameter (fine)") },
+	{  99, _TR("Non-Registered Parameter (coarse)") },
+	{ 100, _TR("Registered Parameter (fine)") },
+	{ 101, _TR("Registered Parameter (coarse)") },
+	{ 120, _TR("All Sound Off") },
+	{ 121, _TR("All Controllers Off") },
+	{ 122, _TR("Local Keyboard (on/off)") },
+	{ 123, _TR("All Notes Off") },
+	{ 124, _TR("Omni Mode Off") },
+	{ 125, _TR("Omni Mode On") },
+	{ 126, _TR("Mono Operation") },
+	{ 127, _TR("Poly Operation") },
 
-	{   0, NULL }
+	{   0, nullptr }
 };
 
 static QHash<unsigned short, QString> g_controllerNames;
@@ -585,9 +588,8 @@ const QString& qmidictlMidiControl::controllerName (
 	if (g_controllerNames.isEmpty()) {
 		// Pre-load controller-names hash table...
 		for (int i = 0; g_aControllerNames[i].name; ++i) {
-			g_controllerNames.insert(
-				g_aControllerNames[i].controller,
-				QObject::tr(g_aControllerNames[i].name));
+			g_controllerNames.insert(g_aControllerNames[i].controller,
+				QObject::tr(g_aControllerNames[i].name, "controllerName"));
 		}
 	}
 
